unit Scores;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, FormSettings, DB;

type
  TScoreForm = class(TForm)
    TopPanel: TPanel;
    ScoreBut: TButton;
    SpreadBut: TButton;
    Memo1: TMemo;
    PasteBut: TButton;
    FormSettings1: TFormSettings;
    ListBox1: TListBox;
    Label1: TLabel;
    WeekEdit: TEdit;
    procedure PasteButClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpreadButClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Teams      : TStringList;
    procedure  LoadTeams;
    function   HasTeamAbbrev(s: TStringList): boolean;
    function   GetTeamAbbrev(s: TStringList): string;
    procedure  Enter_Spread(const Team1, Team2, v: string);
  end;

var
  ScoreForm: TScoreForm;

implementation

uses Pool_Mod_Unit;

{$R *.DFM}

function IsSpreadNum(const s: string): boolean;
var
   i : integer;
begin
   Result := True;
   for i := 1 to Length(s) do
      if not (s[i] in ['0'..'9', '''']) then Result := False;
end;

function ConvertSpreadTick(const s: string): string;
begin
   Result := s;
   if Copy(s, Length(s), 1) = '''' then
      Result := Copy(s, 1, Length(s)-1) + '.5';
end;

procedure Fill_List(const Line: string; sl: TStringList);
const
   WHITESPACE = [' ', #9];
var
   tmp     : string;
   i       : integer;
   CurData : string;
   c       : char;
begin
   sl.Clear;

   tmp := Trim(Line);
   i := 1;

   while (i <= Length(tmp)) do begin
      c := tmp[i];
      if not (c in WHITESPACE) then begin
         CurData := CurData + c
      end else begin
         if CurData<>'' then sl.Add(CurData);
         CurData := '';
      end;

      i := i + 1;
   end;
   if CurData<>'' then sl.Add(CurData);
end;

procedure TScoreForm.PasteButClick(Sender: TObject);
begin
   Memo1.PasteFromClipboard;
end;

procedure TScoreForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   FormSettings1.SaveSettings;
end;

type
   TSpreadState = (NeedTeam1, NeedTeam2);

procedure TScoreForm.SpreadButClick(Sender: TObject);
var
   i     : integer;
   Line  : string;
   Team1 : string;
   Team2 : string;
   v     : string;
   s     : TStringList;
   State : TSpreadState;
begin
   LoadTeams;
   s := TStringList.Create;
   ListBox1.Items.Clear;
   v := '0.1';

   State := NeedTeam1;
   for i := 0 to Memo1.Lines.Count-1 do begin
      Line := Memo1.Lines[i];
      Fill_List(Line, s);

      if (State = NeedTeam1) and HasTeamAbbrev(s) then begin
         Team1 := GetTeamAbbrev(s);
         State := NeedTeam2;
         if IsSpreadNum(s[s.Count-1]) then v := '-' + s[s.Count-1];
         continue;
      end;

      if (State = NeedTeam2) then begin
         if HasTeamAbbrev(s) then begin
            Team2 := GetTeamAbbrev(s);
            State := NeedTeam1;
            if IsSpreadNum(s[s.Count-1]) then v := s[s.Count-1];
            v := ConvertSpreadTick(v);
            ListBox1.Items.Add(Team1 + ' ' + Team2 + ' ' + v);
            Enter_Spread(Team1, Team2, v);
            v := '0.1';
            // continue;
         end else begin
            Assert(False, 'No Team Abbreviation on Line 2!'#13#10 + Line);
         end;
      end;
   end;

   s.Free;
end;

procedure TScoreForm.FormCreate(Sender: TObject);
begin
   Teams := TStringList.Create;
end;

procedure TScoreForm.FormDestroy(Sender: TObject);
begin
   Teams.Free;
end;

function TScoreForm.GetTeamAbbrev(s: TStringList): string;
var
   i : integer;
begin
   Result := '';

   for i := 0 to s.Count-1 do
      if Teams.IndexOf(s[i]) <> -1 then
         Result := s[i];
end;

function TScoreForm.HasTeamAbbrev(s: TStringList): boolean;
begin
   Result := GetTeamAbbrev(s) <> '';
end;

procedure TScoreForm.LoadTeams;
var
   s : string;
   i : integer;
begin
   Teams.Clear;

   with Pool_Module.TeamsTable do begin
      Active := True;
      First;

      while not EOF do begin
         s := Pool_Module.TeamsTableTeamShort.AsString;
         i := Pool_Module.TeamsTableTeamID.AsInteger;

         Teams.AddObject(s, TObject(i));

         Next;
      end;
   end;
end;

procedure TScoreForm.FormShow(Sender: TObject);
begin
   with Pool_Module.GlobalsTable do begin
      Active := True;
      First;
      WeekEdit.Text := FieldByName('CurWeek').AsString;
   end;
end;

procedure TScoreForm.Enter_Spread(const Team1, Team2, v: string);
var
   s1, s2 : string;
   f      : boolean;
begin
   with Pool_Module.GamesTable do begin
      Active := True;
      SetRange([WeekEdit.Text], [WeekEdit.Text]);

      f := False;
      First;
      while not EOF do begin
         s1 := UpperCase(Team2);
         s2 := UpperCase(Pool_Module.GamesTableHomeShort.AsString);
         if s1 = s2 then begin
            s1 := UpperCase(Team1);
            s2 := UpperCase(Pool_Module.GamesTableAwayShort.AsString);
            Assert(s1 = s2);

            f := True;
            break;
         end;

         Next;
      end;

      Assert(f, 'Can not find: ' + WeekEdit.Text + ' ' + Team1 + ' ' + Team2);
      Edit;
      Pool_Module.GamesTableSpread.AsString := v;
      Post;

      CancelRange;
   end;
end;

end.
