unit SimWin;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, BenTools, FormSettings, ComCtrls;

type
   TGame = class
      Num      : integer;
      HomeTeam : string;
      AwayTeam : string;
      Spread   : double;
   end;

   TPickArray = array[1..15] of integer;
   TSpreadArray = array[1..15] of double;

   TPlayer = class
      ID       : integer;
      Name     : string;
      Picks    : TPickArray;
      TotWin   : integer;
      TotTie   : integer;
      Score    : double;
      NumRight : integer;
      procedure   Assign(Source: TPlayer);
   end;

   TGameList = class(TOwnList)
   protected
      function GetGame(i: integer): TGame;
   public
      property Game[i: integer]: TGame read GetGame; default;
   end;

   TPlayerList = class(TOwnList)
   protected
      function GetPlayer(i: integer): TPlayer;
   public
      function PlayerByID(ID: integer): TPlayer;
      property Player[i: integer]: TPlayer read GetPlayer; default;
   end;

   TSimWinForm = class;

   TSimThread = class(TThread)
   protected
      procedure   Execute; override;
   public
      MainForm    : TSimWinForm;
   end;

  TSimWinForm = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    WeekEdit: TEdit;
    SimBut: TButton;
    FormSettings1: TFormSettings;
    StatusBar1: TStatusBar;
    ModEdit: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    TimeLabel: TLabel;
    Timer1: TTimer;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Memo1: TMemo;
    TabSheet2: TTabSheet;
    Memo2: TMemo;
    EvalBut: TButton;
    procedure SimButClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
    procedure EvalButClick(Sender: TObject);
  private
    StatIdx    : integer;
    StatMsg    : string;
    st         : TSimThread;
    StopNow    : boolean;
    procedure  SetStatMessage;
    function   GetStat(i: integer): string;
    procedure  SetStat(i: integer; const Value: string);
    procedure  SetThreadStat(i: integer; const Value: string);
    function   GetThreadStat(i: integer): string;
  protected
    StartTime  : DWORD;
    Spreads    : TSpreadArray;
    Num        : integer;
    procedure  LoadGames;
    procedure  LoadPlayers;
  public
    CurWeek    : integer;
    GameList   : TGameList;
    PlayerList : TPlayerList;
    NumGame    : integer;
    procedure  LoadData;
    function   NextSolution(var p: TPickArray): boolean;
    function   TestOne(p, bestp: TPlayer): boolean;
    procedure  Evaluate;
    procedure  RunSim;
    procedure  ThreadDone(Sender: TObject);
    procedure  Fill_All_Players;
    property   Stat[i: integer]: string read GetStat write SetStat;
    property   ThreadStat[i: integer]: string  read GetThreadStat write SetThreadStat;
  end;

var
  SimWinForm: TSimWinForm;

implementation

uses Pool_Mod_Unit, ProbTab;

{$R *.DFM}

{ TPlayer }

procedure TPlayer.Assign(Source: TPlayer);
begin
   ID     := Source.ID;
   Name   := Source.Name;
   Picks  := Source.Picks;
   Score  := Source.Score;
   TotWin := Source.TotWin;
   TotTie := Source.TotTie;
end;

{ TGameList }

function TGameList.GetGame(i: integer): TGame;
begin
   Result := Items[i];
end;

{ TPlayerList }

function TPlayerList.GetPlayer(i: integer): TPlayer;
begin
   Result := Items[i];
end;

function TPlayerList.PlayerByID(ID: integer): TPlayer;
var
   i : integer;
begin
   Result := nil;
   for i := 0 to Count-1 do
      if Player[i].ID = ID then Result := Player[i];
end;

{ TSimWinForm }

procedure TSimWinForm.LoadGames;
var
   g    : TGame;
begin
   Stat[3] := 'Loading Games';
   GameList.ClearItems;
   with Pool_Module, Pool_Module.GamesTable do begin
      Active := True;
      FindKey([Self.CurWeek]);

      while not EOF do begin
         g := TGame.Create;
         g.Num      := GamesTableGameNum.AsInteger;
         g.HomeTeam := GamesTableHomeTeamName.AsString;
         g.AwayTeam := GamesTableAwayTeamName.AsString;
         g.Spread   := GamesTableSpread.AsFloat;
         GameList.Add(g);
         
         Spreads[g.Num] := g.Spread;

         Next;
         if GamesTableWeek.AsInteger <> Self.CurWeek then break;
      end;
   end;

   NumGame := GameList.Count;
end;

procedure TSimWinForm.LoadPlayers;
var
   p   : TPlayer;
   pid : integer;
   gid : integer;
begin
   Stat[3] := 'Loading Players';
   PlayerList.ClearItems;
   with Pool_Module, Pool_Module.PicksTable do begin
      Active := True;
      FindKey([Self.CurWeek]);

      while not EOF do begin
         pid := PicksTablePlayerID.AsInteger;
         p := PlayerList.PlayerByID(pid);
         if p = nil then begin
            p := TPlayer.Create;
            p.ID   := pid;
            p.Name := PicksTablePlayerName.AsString;
            PlayerList.Add(p);
         end;
         gid := PicksTableGameID.AsInteger;
         p.Picks[gid] := PicksTablePick.AsInteger;

         Next;
         if PicksTableWeek.AsInteger <> Self.CurWeek then break;
      end;
   end;
end;

procedure TSimWinForm.LoadData;
begin
   Screen.Cursor := crHourGlass;
   LoadGames;
   LoadPlayers;
   Screen.Cursor := crDefault;
end;

procedure TSimWinForm.SimButClick(Sender: TObject);
begin
   CurWeek := StrToInt(WeekEdit.Text);
   LoadData;
   SimBut.Enabled := False;
   Timer1.Enabled := True;
   StartTime := GetTickCount;

   st             := TSimThread.Create(True);
   st.MainForm    := Self;
   st.Priority    := tpLowest;
   st.OnTerminate := ThreadDone;
   st.Resume;
end;

function TSimWinForm.TestOne(p, bestp: TPlayer): boolean;
var
   i : integer;
   s : string;
begin
   Evaluate;
   Result := False;

   // Remember the best one
   if p.Score > bestp.Score then begin
      Result := True;
      bestp.Assign(p);
      s := '';
      for i := 1 to NumGame do s := s + IntToStr(bestp.Picks[i]) + ' ';
      ThreadStat[4] := Format('Iter %5d:  Score = %1.5n   Win = %5d   Tie = %5d   %s',
         [Num, bestp.Score, bestp.TotWin, bestp.TotTie, s]);

      st.Synchronize(Fill_All_Players);
   end;

   Num := Num + 1;
   ThreadStat[0] := Format('Iter %d', [Num]);
end;

procedure TSimWinForm.RunSim;
var
   p       : TPlayer;
   bestp   : TPlayer;
   i, j    : integer;
   Orig    : TPickArray;
   b       : boolean;
   Changed : boolean;
   Num1, Num2 : integer;
begin
   bestp := TPlayer.Create;
   bestp.Score := -1;

   p := PlayerList.PlayerByID(StrToInt(ModEdit.Text));
   Assert(p <> nil, 'No Player for that ID!');

   for i := 1 to NumGame do
      p.Picks[i] := 1;

   Num := 0;
   Memo1.Lines.Clear;

   // 1-opt
   Num1 := 0;
   repeat
      Num1 := Num1 + 1;
      ThreadStat[3] := Format('1-Opt: %d', [Num1]);
      Changed := False;
      for i := 1 to NumGame do begin
         ThreadStat[1] := Format('i = %d', [i]);
         if StopNow then exit;
         Orig := p.Picks;

         p.Picks[i] := 3 - p.Picks[i];

         b := TestOne(p, bestp);
         if not b then p.Picks := Orig;
         Changed := Changed or b;
      end;
   until not Changed;

   // 2-opt
   Num2 := 0;
   repeat
      Num2 := Num2 + 1;
      ThreadStat[3] := Format('2-Opt: %d', [Num2]);
      Changed := False;
      for j := 1 to NumGame do begin
         ThreadStat[1] := Format('j = %d', [j]);
         for i := 1 to NumGame do begin
            ThreadStat[2] := Format('i = %d', [i]);
            if StopNow then exit;
            Orig := p.Picks;

            p.Picks[i] := 3 - p.Picks[i];
            if i<>j then p.Picks[j] := 3 - p.Picks[j];

            b := TestOne(p, bestp);
            if not b then p.Picks := Orig;
            Changed := Changed or b;
         end;
      end;
   until not Changed;

   // Display the Results

   bestp.Free;
end;

procedure TSimWinForm.FormCreate(Sender: TObject);
begin
   GameList := TGameList.Create;
   PlayerList := TPlayerList.Create;
end;

function TSimWinForm.GetStat(i: integer): string;
begin
   Result := StatusBar1.Panels[i].Text;
end;

procedure TSimWinForm.SetStat(i: integer; const Value: string);
begin
   StatusBar1.Panels[i].Text := Value;
   Update;
end;

procedure TSimWinForm.FormDestroy(Sender: TObject);
begin
   GameList.Free;
   PlayerList.Free;
end;

function TSimWinForm.NextSolution(var p: TPickArray): boolean;
var
   i : integer;
begin
   i := 1;

   while (i <= NumGame) and (p[i] = 2) do begin
      p[i] := 1;
      i := i + 1;
   end;
   if (i <= NumGame) then p[i] := 2;

   Result := True;
   for i := 1 to NumGame do
      Result := Result and (p[i] = 1);
end;

procedure TSimWinForm.Evaluate;
var
   i         : integer;
   g         : integer;
   Winners   : TPickArray;
   BestRight : integer;
   NumTie    : integer;
   Prob      : double;
begin
   for i := 0 to PlayerList.Count-1 do with PlayerList[i] do begin
      Score  := 0;
      TotWin := 0;
      TotTie := 0;
   end;

   // Loop over all combinations of winners
   for i := 1 to NumGame do
      Winners[i] := 1;

   repeat
      // Calculate the probability of this Win combination
      Prob := 1;
      for i := 1 to NumGame do begin
         if Winners[i] = 1 then
            Prob := Prob * GetProb(Spreads[i])
         else
            Prob := Prob * (1.0 - GetProb(Spreads[i]));
      end;

      // Find the number of correct picks for each player
      for i := 0 to PlayerList.Count-1 do with PlayerList[i] do begin
         NumRight := 0;
         for g := 1 to NumGame do
            if Winners[g] = Picks[g] then
               NumRight := NumRight + 1;
      end;

      // Determine Best Score (and number of ties)
      BestRight := -1;
      NumTie := 0;
      for i := 0 to PlayerList.Count-1 do with PlayerList[i] do begin
         if NumRight > BestRight then begin
            BestRight := NumRight;
            NumTie := 1;
         end else begin
            if NumRight = BestRight then
               NumTie := NumTie + 1;
         end;
      end;

      // Find which player had the best score (or tied)
      for i := 0 to PlayerList.Count-1 do with PlayerList[i] do begin
         if NumRight = BestRight then begin
            if NumTie = 1 then begin
               TotWin := TotWin + 1;
               Score := Score + Prob;
            end else begin
               TotTie := TotTie + 1;
               Score := Score + (Prob / NumTie);
            end;
         end;
      end;

   until NextSolution(Winners);
end;

procedure TSimWinForm.SetThreadStat(i: integer; const Value: string);
begin
   StatIdx := i;
   StatMsg := Value;
   st.Synchronize(SetStatMessage);
end;

function TSimWinForm.GetThreadStat(i: integer): string;
begin
   Result := Stat[i];
end;

procedure TSimWinForm.SetStatMessage;
begin
   if StatIdx = 4 then Memo1.Lines.Add(StatMsg)
      else Stat[StatIdx] := StatMsg;
end;

procedure TSimWinForm.ThreadDone(Sender: TObject);
begin
   SimBut.Enabled := True;
   st.Free;
   st := nil;
   Timer1.Enabled := False;
end;

function SortFunc(Item1, Item2: Pointer): Integer;
var
   p1, p2 : TPlayer;
begin
   p1 := TPlayer(Item1);
   p2 := TPlayer(Item2);
   Result := Round((p2.Score - p1.Score) * 100000);
end;

procedure TSimWinForm.Fill_All_Players;
var
   i, j : integer;
   s, t : string;
begin
   Memo2.Lines.Clear;
   PlayerList.Sort(SortFunc);
   for i := 0 to PlayerList.Count-1 do with PlayerList[i] do begin
      s := Format('%3d  %-15s  Wins: %5d  Ties:  %5d  Score: %1.5n  ',
         [i+1, Name, TotWin, TotTie, Score]);
      t := '';
      for j := 1 to NumGame do t := t + IntToStr(Picks[j]) + ' ';
      Memo2.Lines.Add(s + t);
   end;
end;

{ TSimThread }

procedure TSimThread.Execute;
begin
   MainForm.RunSim;
end;

procedure TSimWinForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   if st<>nil then begin
      StopNow := True;
      st.WaitFor;
   end;
   FormSettings1.SaveSettings;
end;

procedure TSimWinForm.Timer1Timer(Sender: TObject);
var
   NumSec : double;
begin
   NumSec := (GetTickCount - StartTime) / 1000.0;
   TimeLabel.Caption := Format('Seconds = %1.1n', [NumSec]);
end;

procedure TSimWinForm.EvalButClick(Sender: TObject);
begin
   CurWeek := StrToInt(WeekEdit.Text);
   LoadData;
   Evaluate;
   Fill_All_Players;
   PageControl1.ActivePage := TabSheet2;
end;

end.
